package com.jh.boot.attachment;

import java.io.InputStream;
import java.util.List;

import org.springframework.web.multipart.MultipartFile;

public interface AttachmentService {

    Attachment saveAttachment(Attachment attachment, InputStream content);

    Attachment saveAttachment(Attachment attachment);

    List<Attachment> fetchAttachments(Long unitId, String objectType, Long objectId);

    Attachment fetchAttachment(Long attachmentId);

    InputStream fetchAttachmentBody(Attachment attachment);

    void deleteAttachment(Long attachmentId);

    
    // default methods
    
    public default Attachment addAttachment(String objectType, Long objectId, MultipartFile file) {
        return addAttachment(null, objectType, objectId, file);
    }

    public default Attachment addAttachment(Long unitId, String objectType, Long objectId, MultipartFile file) {
        Attachment a = new Attachment();
        a.setFilename(file.getOriginalFilename());
        a.setSize(file.getSize());
        a.setObjectId(objectId);
        a.setObjectType(objectType);
        try {
            saveAttachment(a, file.getInputStream());
        } catch (Exception e) {
            throw new RuntimeException("Cannot upload attachment", e);
        }

        return a;
    }

    public default Attachment syncAttachment(String objectType, Long objectId, Attachment currentAttachment, Attachment newAttachment, MultipartFile file) {
        return syncAttachment(null, objectType, objectId, currentAttachment, newAttachment, file);
    }

    public default Attachment syncAttachment(Long unitId, String objectType, Long objectId, Attachment currentAttachment, Attachment newAttachment, MultipartFile file) {
        Attachment a = null;
        if (currentAttachment != null && newAttachment == null) {
            //delete current attachment
            deleteAttachment(currentAttachment.getId());
        }
        if (file != null) {
            a = addAttachment(unitId, objectType, objectId, file);
        } else {
            a = currentAttachment;
        }
        return a;
    }

    
    
}
